use leptos::*;

pub mod schemas;

cfg_if::cfg_if! {
    if #[cfg(feature = "ssr")] {
        use once_cell::sync::Lazy;
        use surrealdb::engine::remote::ws::{Client, Ws};
        use surrealdb::opt::auth::Root;
        use surrealdb::Surreal;

        pub static DB: Lazy<Surreal<Client>> = Lazy::new(Surreal::init);
    }
}

#[cfg(feature = "ssr")]
pub async fn connect() -> Result<(), ServerFnError> {
    DB.connect::<Ws>("localhost:80").await?;

    DB.signin(Root {
        username: "root",
        password: "root",
    })
    .await?;

    DB.use_ns("wrb").use_db("timings").await?;

    Ok(())
}

#[server]
pub async fn get_participants() -> Result<Vec<schemas::Participant>, ServerFnError> {
    let participant_records: Vec<schemas::ParticipantRecord> = DB.select("participant").await?;

    let mut participants: Vec<schemas::Participant> = vec![];

    participant_records.iter().for_each(|participant| {
        participants.push(schemas::Participant {
            id: participant.id.id.to_string(),
            name: participant.name.clone(),
            group: participant.group.clone(),
            events: participant.events.clone(),
        })
    });

    Ok(participants)
}

pub async fn init_participants() -> Result<bool, ServerFnError> {
    let participants = get_participants().await?;

    let participants_signal = create_rw_signal(participants);
    provide_context(participants_signal);

    Ok(true)
}
